#!/usr/bin/perl

package HPSMSOAP;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(soapdata soapfault soapstatus
	soapnew soapkeyslist soapretrieve soapretrieveall);

use strict;
use warnings;
use SOAP::Lite; # +trace => 'debug';
use MIME::Base64;

sub soapdata {
    my ($som, $root, $num) = @_;
    sdindent($som, $root, $num, 0);
}

# private implementation
sub sdindent {
    my ($som, $root, $num, $indent) = @_;
    my $ref = $som->valueof($root);
    my @keys = sort(keys(%$ref));
    my $indstr = ' ' x int($indent);
    foreach my $k (@keys) {
	my $count = 0;
	foreach my $v ($som->valueof("$root/$k")) {
	    my $kk = $k;
	    $count += 1;
	    if (! defined $v) {
		$kk .= "[$count]" if ($num);
		print "$indstr$kk: (undefined)\n";
	    } elsif (ref($v) eq 'HASH') {
		print "$indstr$kk: (\n";
		sdindent($som, "$root/$k", 1, $indent + 2);
		print "$indstr$kk: )\n";
	    } elsif ($v eq '') {
		$kk .= "[$count]" if ($num);
		print "$indstr$kk: (empty)\n";
	    } else {
		$kk .= "[$count]" if ($num);
		my @ls = split(/\n/, $v);
		my $lcount = 0;
		foreach my $l (@ls) {
		    $lcount += 1;
		    $kk .= "($lcount)" if ($#ls > 0);
		    print "$indstr$kk: $l\n";
		    $kk =~ s/\(\d+\)$//;
		}
	    }
	}
    }
}

sub soapfault {
    my $som = $_[0];
    if ($som->fault) {
	my $code = $som->faultcode;
	my $actor = $som->faultactor;
	my $string = $som->faultstring;
	my $detail = $som->faultdetail;
	my $out = "Fault\n";
	$out .= "\tCode\t$code\n" if ($code);
	$out .= "\tActor\t$actor\n" if ($actor);
	$out .= "\tString\t$string\n" if ($string);
	$out .= "\tDetail\t$detail\n" if ($detail);
	die $out;
    }
}

sub soapstatus {
    my ($som) = @_;
    my $attr = $som->dataof('/Envelope/Body/[1]')->attr;
    my $status = $attr->{'status'};
    my $message = $attr->{'message'};
    print "SOAP $status: $message\n";
}

our %typewsdl =
    ('Change' => 'ChangeManagement',
     'ChangeIIA' => 'ChangeManagement',
     'ChangeOperatorInformation' => 'ChangeManagement',
     'ChangeRC' => 'ChangeManagement',
     'ChangeTask' => 'ChangeManagement',
     'ChangeTaskOperatorInformation' => 'ChangeManagement',
     'ChangeTaskRC' => 'ChangeManagement',
     'Company' => 'ConfigurationManagement',
     'Computer' => 'ConfigurationManagement',
     'Contact' => 'ConfigurationManagement',
     'Department' => 'ConfigurationManagement',
     'Device' => 'ConfigurationManagement',
     'DisplayDevice' => 'ConfigurationManagement',
     'Furnishing' => 'ConfigurationManagement',
     'GlobalLists' => 'FSCManagement',
     'HandHeldDevice' => 'ConfigurationManagement',
     'Inbox' => 'FSCManagement',
     'Incident' => 'IncidentManagement',
     'IncidentwithoutJournalUpdates' => 'IncidentManagement',
     'InstalledSoftware' => 'ConfigurationManagement',
     'Interaction' => 'ServiceDesk',
     'KMAttachments' => 'KnowledgeManagement',
     'Knowledge' => 'KnowledgeManagement',
     'Location' => 'ConfigurationManagement',
     'MainFrame' => 'ConfigurationManagement',
     'Model' => 'ConfigurationManagement',
     'NetworkDevice' => 'ConfigurationManagement',
     'OfficeElectronic' => 'ConfigurationManagement',
     'Operator' => 'FSCManagement',
     'Problem' => 'ProblemManagement',
     'SoftwareLicense' => 'ConfigurationManagement',
     'StorageDevice' => 'ConfigurationManagement',
     'TelecommunicationDevice' => 'ConfigurationManagement',
     'Vendor' => 'ConfigurationManagement',
    );

sub soapnew {
    my ($site, $user, $password, $type, $action) = @_;
    my $wsdl = $typewsdl{$type};
    die "$type not known to WSDL\n" unless (length($wsdl) > 0);
    my $value = SOAP::Lite->new(
	proxy => "$site/ws",
	service => "$site/$wsdl.wsdl",
	default_ns => $site);
    my $auth = encode_base64("$user:$password", '');
    $value->transport->http_request->header('Authorization' => "Basic $auth");
    $value->on_action(sub{"\"$action\"";});
    return $value;
}

sub soapkeyslist {
    my ($site, $user, $password, $type) = @_;
    my $wsdl = $typewsdl{$type};
    die "$type not known to WSDL\n" unless (length($wsdl) > 0);
    my $soap = soapnew($site, $user, $password, $type, 'RetrieveKeysList');
    my $keys = SOAP::Data->name('keys' => '')->type('');
    my $instance = SOAP::Data->name('instance' => '')->type('');
    my $model = SOAP::Data->name('model' =>
				 \SOAP::Data->value($keys, $instance));
    my $som = $soap->call('Retrieve' . $type . 'KeysList', $model);
    soapfault($som);
    my $keyname;
    my @values = ();
    foreach my $i ($som->valueof('//Retrieve' . $type . 'KeysListResponse/keys')) {
	my @kv = %$i;
	$keyname = $kv[0];
	push(@values, $kv[1]);
    }
    return ($keyname, @values);
}

sub soapretrieve {
    my ($site, $user, $password, $type, $keyname, $key) = @_;
    my $wsdl = $typewsdl{$type};
    die "$type not known to WSDL\n" unless (length($wsdl) > 0);
    my $soap = soapnew($site, $user, $password, $type, 'Retrieve');
    my $keyvalue = SOAP::Data->name($keyname => $key);
    my $keys = SOAP::Data->name('keys' => '')->type('');
    my $instance = SOAP::Data->name('instance' => \$keyvalue)->type('');
    my $model = SOAP::Data->name('model' =>
				 \SOAP::Data->value($keys, $instance));
    my $som = $soap->call('Retrieve' . $type, $model);
    soapfault($som);
    return $som;
}

sub soapretrieveall {
    my ($site, $user, $password, $type) = @_;
    my $wsdl = $typewsdl{$type};
    die "$type not known to WSDL\n" unless (length($wsdl) > 0);
    my ($name, @keys) = soapkeyslist($site, $user, $password, $type);
    my @result = ();
    foreach my $key (@keys) {
	push (@result, soapretrieve($site, $user, $password, $type, $name, $key));
    }
    return @result;
}

1;
